from typing import Generic, TypeVar

from rainbowadn.core import *
from rainbowadn.flow.core import *

__all__ = ('ListBridge',)

Element = TypeVar('Element')
Out = TypeVar('Out')


class ListBridge(
    Reducer[Element, Out],
    Generic[Element, Out]
):
    def __init__(self, target: list[Element]):
        assert isinstance(target, list)
        self.target = target

    async def reduce(self, reduce: Reduce[Element, Out]) -> Out:
        assert isinstance(reduce, Reduce)
        if not self.target:
            return reduce.initial
        elif len(self.target) == 1:
            return await reduce.reduce(reduce.initial, self.target[0])
        else:
            middle = len(self.target) // 2
            left_list: list[Element] = self.target[:middle]
            right_list: list[Element] = self.target[middle:]
            left_bridge: ListBridge[Element, Out] = ListBridge(left_list)
            right_bridge: ListBridge[Element, Out] = ListBridge(right_list)
            left: Out
            right: Out
            left, right = await gather(
                left_bridge.reduce(reduce),
                right_bridge.reduce(reduce),
            )
            return reduce.merge(left, right)

    def loose(self) -> Reducer[Element, Out]:
        return self
