from typing import Iterable

from rainbowadn.collection.comparison import *
from rainbowadn.core import *
from ._flowstandard import *
from ._flowtransaction import *

__all__ = ('FlowBank',)


class FlowBank(StaticMentionable, RecursiveMentionable):
    @classmethod
    def from_bytes(cls, source: bytes, resolver: HashResolver) -> 'FlowBank':
        return FlowBank(
            FlowStandardFactory.of(FlowCoin.factory(), HashComparator(Fail())).from_bytes(
                source[:HashPoint.HASH_LENGTH], resolver
            ),
            FlowStandardFactory.of(FlowCoin.factory(), HashComparator(Fail())).from_bytes(
                source[HashPoint.HASH_LENGTH:], resolver
            ),
        )

    def points(self) -> Iterable[HashPoint]:
        return [*self.minted.points(), *self.used.points()]

    def __bytes__(self):
        return bytes(self.minted) + bytes(self.used)

    def __init__(
            self,
            minted: FlowStandard[FlowCoin],
            used: FlowStandard[FlowCoin],
    ):
        assert isinstance(minted, FlowStandard)
        assert isinstance(used, FlowStandard)
        self.minted = minted
        self.used = used

    @classmethod
    def empty(cls) -> 'FlowBank':
        return FlowBank(
            FlowStandardFactory.empty(FlowCoin.factory(), HashComparator(Fail())),
            FlowStandardFactory.empty(FlowCoin.factory(), HashComparator(Fail())),
        )

    async def str(self, tab: int) -> str:
        assert isinstance(tab, int)
        return f'(' \
               f'{tabulate(tab + 1)}bank' \
               f'{tabulate(tab + 1)}(minted)' \
               f'{tabulate(tab + 1)}(used)' \
               f'{tabulate(tab)})'
